@@ -1,6 +1,7 @@
 use atty::Stream;
+use base64::decode as base64_decode;
 use chrono::{TimeZone, Utc};
-use clap::{arg_enum, crate_authors, crate_version, App, Arg, ArgMatches, SubCommand};
+use clap::{arg_enum, crate_authors, crate_version, App, AppSettings, Arg, ArgMatches, SubCommand};
 use jsonwebtoken::errors::{ErrorKind, Result as JWTResult};
 use jsonwebtoken::{
     dangerous_insecure_decode, decode, encode, Algorithm, DecodingKey, EncodingKey, Header,
@@ -11,9 +12,6 @@ use serde_json::{from_str, to_string_pretty, Value};
 use std::collections::BTreeMap;
 use std::process::exit;
 use std::{fs, io};
-use term_painter::Attr::*;
-use term_painter::Color::*;
-use term_painter::ToStyle;
 
 #[derive(Debug, Serialize, Deserialize, PartialEq)]
 struct PayloadItem(String, Value);
@@ -28,6 +26,7 @@ struct TokenOutput {
 }
 
 arg_enum! {
+    #[allow(clippy::clippy::upper_case_acronyms)]
     #[derive(Debug, PartialEq)]
     enum SupportedAlgorithms {
         HS256,
@@ -36,12 +35,16 @@ arg_enum! {
         RS256,
         RS384,
         RS512,
+        PS256,
+        PS384,
+        PS512,
         ES256,
         ES384,
     }
 }
 
 arg_enum! {
+    #[allow(clippy::clippy::upper_case_acronyms)]
     enum SupportedTypes {
         JWT
     }
@@ -50,15 +53,12 @@ arg_enum! {
 #[derive(Debug, PartialEq)]
 enum OutputFormat {
     Text,
-    JSON,
+    Json,
 }
 
 impl PayloadItem {
     fn from_string(val: Option<&str>) -> Option<PayloadItem> {
-        match val {
-            Some(item) => Some(PayloadItem::split_payload_item(item)),
-            None => None,
-        }
+        val.map(|item| PayloadItem::split_payload_item(item))
     }
 
     fn from_string_with_name(val: Option<&str>, name: &str) -> Option<PayloadItem> {
@@ -132,6 +132,9 @@ impl SupportedAlgorithms {
             "RS256" => SupportedAlgorithms::RS256,
             "RS384" => SupportedAlgorithms::RS384,
             "RS512" => SupportedAlgorithms::RS512,
+            "PS256" => SupportedAlgorithms::PS256,
+            "PS384" => SupportedAlgorithms::PS384,
+            "PS512" => SupportedAlgorithms::PS512,
             "ES256" => SupportedAlgorithms::ES256,
             "ES384" => SupportedAlgorithms::ES384,
             _ => SupportedAlgorithms::HS256,
@@ -153,6 +156,7 @@ fn config_options<'a, 'b>() -> App<'a, 'b> {
         .about("Encode and decode JWTs from the command line. RSA and ECDSA encryption currently only supports keys in DER format")
         .version(crate_version!())
         .author(crate_authors!())
+        .setting(AppSettings::ArgRequiredElseHelp)
         .subcommand(
             SubCommand::with_name("encode")
                 .about("Encode new JWTs")
@@ -235,7 +239,7 @@ fn config_options<'a, 'b>() -> App<'a, 'b> {
                         .long("no-iat")
                 ).arg(
                     Arg::with_name("secret")
-                        .help("the secret to sign the JWT with. Can be prefixed with @ to read from a binary file")
+                        .help("the secret to sign the JWT with. Prefix with @ to read from a file or b64: to use base-64 encoded bytes")
                         .takes_value(true)
                         .long("secret")
                         .short("S")
@@ -264,7 +268,7 @@ fn config_options<'a, 'b>() -> App<'a, 'b> {
                         .long("iso8601")
                 ).arg(
                     Arg::with_name("secret")
-                        .help("the secret to validate the JWT with. Can be prefixed with @ to read from a binary file")
+                        .help("the secret to validate the JWT with. Prefix with @ to read from a file or b64: to use base-64 encoded bytes")
                         .takes_value(true)
                         .long("secret")
                         .short("S")
@@ -274,12 +278,16 @@ fn config_options<'a, 'b>() -> App<'a, 'b> {
                         .help("render decoded JWT as JSON")
                         .long("json")
                         .short("j"),
+                ).arg(
+                    Arg::with_name("ignore_exp")
+                        .help("Ignore token expiration date (`exp` claim) during validation.")
+                        .long("ignore-exp")
                 ),
         )
 }
 
 fn is_timestamp_or_duration(val: String) -> Result<(), String> {
-    match i64::from_str_radix(&val, 10) {
+    match val.parse::<i64>() {
         Ok(_) => Ok(()),
         Err(_) => match parse_duration::parse(&val) {
             Ok(_) => Ok(()),
@@ -291,9 +299,7 @@ fn is_timestamp_or_duration(val: String) -> Result<(), String> {
 }
 
 fn is_payload_item(val: String) -> Result<(), String> {
-    let split: Vec<&str> = val.split('=').collect();
-
-    match split.len() {
+    match val.split('=').count() {
         2 => Ok(()),
         _ => Err(String::from(
             "payloads must have a key and value in the form key=value",
@@ -315,6 +321,9 @@ fn translate_algorithm(alg: SupportedAlgorithms) -> Algorithm {
         SupportedAlgorithms::RS256 => Algorithm::RS256,
         SupportedAlgorithms::RS384 => Algorithm::RS384,
         SupportedAlgorithms::RS512 => Algorithm::RS512,
+        SupportedAlgorithms::PS256 => Algorithm::PS256,
+        SupportedAlgorithms::PS384 => Algorithm::PS384,
+        SupportedAlgorithms::PS512 => Algorithm::PS512,
         SupportedAlgorithms::ES256 => Algorithm::ES256,
         SupportedAlgorithms::ES384 => Algorithm::ES384,
     }
@@ -328,14 +337,6 @@ fn create_header(alg: Algorithm, kid: Option<&str>) -> Header {
     header
 }
 
-fn create_validations(alg: Algorithm) -> Validation {
-    Validation {
-        leeway: 1000,
-        algorithms: vec![alg],
-        ..Default::default()
-    }
-}
-
 fn slurp_file(file_name: &str) -> Vec<u8> {
     fs::read(file_name).unwrap_or_else(|_| panic!("Unable to read file {}", file_name))
 }
@@ -343,9 +344,23 @@ fn slurp_file(file_name: &str) -> Vec<u8> {
 fn encoding_key_from_secret(alg: &Algorithm, secret_string: &str) -> JWTResult<EncodingKey> {
     match alg {
         Algorithm::HS256 | Algorithm::HS384 | Algorithm::HS512 => {
-            Ok(EncodingKey::from_secret(secret_string.as_bytes()))
+            if secret_string.starts_with('@') {
+                let secret = slurp_file(&secret_string.chars().skip(1).collect::<String>());
+                Ok(EncodingKey::from_secret(&secret))
+            } else if secret_string.starts_with("b64:") {
+                Ok(EncodingKey::from_secret(
+                    &base64_decode(&secret_string.chars().skip(4).collect::<String>()).unwrap(),
+                ))
+            } else {
+                Ok(EncodingKey::from_secret(secret_string.as_bytes()))
+            }
         }
-        Algorithm::RS256 | Algorithm::RS384 | Algorithm::RS512 => {
+        Algorithm::RS256
+        | Algorithm::RS384
+        | Algorithm::RS512
+        | Algorithm::PS256
+        | Algorithm::PS384
+        | Algorithm::PS512 => {
             let secret = slurp_file(&secret_string.chars().skip(1).collect::<String>());
 
             match secret_string.ends_with(".pem") {
@@ -361,7 +376,6 @@ fn encoding_key_from_secret(alg: &Algorithm, secret_string: &str) -> JWTResult<E
                 false => Ok(EncodingKey::from_ec_der(&secret)),
             }
         }
-        _ => panic!("Unsupported algorithm!"),
     }
 }
 
@@ -371,9 +385,24 @@ fn decoding_key_from_secret(
 ) -> JWTResult<DecodingKey<'static>> {
     match alg {
         Algorithm::HS256 | Algorithm::HS384 | Algorithm::HS512 => {
-            Ok(DecodingKey::from_secret(secret_string.as_bytes()).into_static())
+            if secret_string.starts_with('@') {
+                let secret = slurp_file(&secret_string.chars().skip(1).collect::<String>());
+                Ok(DecodingKey::from_secret(&secret).into_static())
+            } else if secret_string.starts_with("b64:") {
+                Ok(DecodingKey::from_secret(
+                    &base64_decode(&secret_string.chars().skip(4).collect::<String>()).unwrap(),
+                )
+                .into_static())
+            } else {
+                Ok(DecodingKey::from_secret(secret_string.as_bytes()).into_static())
+            }
         }
-        Algorithm::RS256 | Algorithm::RS384 | Algorithm::RS512 => {
+        Algorithm::RS256
+        | Algorithm::RS384
+        | Algorithm::RS512
+        | Algorithm::PS256
+        | Algorithm::PS384
+        | Algorithm::PS512 => {
             let secret = slurp_file(&secret_string.chars().skip(1).collect::<String>());
 
             match secret_string.ends_with(".pem") {
@@ -389,7 +418,6 @@ fn decoding_key_from_secret(
                 false => Ok(DecodingKey::from_ec_der(&secret).into_static()),
             }
         }
-        _ => panic!("Unsupported algorithm!"),
     }
 }
 
@@ -449,11 +477,7 @@ fn encode_token(matches: &ArgMatches) -> JWTResult<String> {
     maybe_payloads.append(&mut custom_payloads.unwrap_or_default());
     maybe_payloads.append(&mut custom_payload.unwrap_or_default());
 
-    let payloads = maybe_payloads
-        .into_iter()
-        .filter(Option::is_some)
-        .map(Option::unwrap)
-        .collect();
+    let payloads = maybe_payloads.into_iter().flatten().collect();
     let Payload(claims) = Payload::from_payloads(payloads);
 
     encoding_key_from_secret(&algorithm, matches.value_of("secret").unwrap())
@@ -471,7 +495,7 @@ fn decode_token(
         matches.value_of("algorithm").unwrap(),
     ));
     let secret = match matches.value_of("secret").map(|s| (s, !s.is_empty())) {
-        Some((secret, true)) => Some(decoding_key_from_secret(&algorithm, &secret)),
+        Some((secret, true)) => Some(decoding_key_from_secret(&algorithm, secret)),
         _ => None,
     };
     let jwt = matches
@@ -493,7 +517,13 @@ fn decode_token(
         .trim()
         .to_owned();
 
-    let secret_validator = create_validations(algorithm);
+    let secret_validator = Validation {
+        leeway: 1000,
+        algorithms: vec![algorithm],
+        validate_exp: !matches.is_present("ignore_exp"),
+        ..Default::default()
+    };
+
     let token_data = dangerous_insecure_decode::<Payload>(&jwt).map(|mut token| {
         if matches.is_present("iso_dates") {
             token.claims.convert_timestamps();
@@ -509,7 +539,7 @@ fn decode_token(
         },
         token_data,
         if matches.is_present("json") {
-            OutputFormat::JSON
+            OutputFormat::Json
         } else {
             OutputFormat::Text
         },
@@ -527,10 +557,7 @@ fn print_encoded_token(token: JWTResult<String>) {
             exit(0);
         }
         Err(err) => {
-            eprintln!(
-                "{}",
-                Red.bold().paint("Something went awry creating the jwt\n")
-            );
+            bunt::eprintln!("{$red+bold}Something went awry creating the jwt{/$}\n");
             eprintln!("{}", err);
             exit(1);
         }
@@ -542,88 +569,61 @@ fn print_decoded_token(
     token_data: JWTResult<TokenData<Payload>>,
     format: OutputFormat,
 ) {
-    let should_validate_exp = if let Ok(token) = &token_data {
-        token.claims.0.contains_key("exp")
-    } else {
-        false
-    };
-
     if let Err(err) = &validated_token {
         match err.kind() {
             ErrorKind::InvalidToken => {
-                println!("{}", Red.bold().paint("The JWT provided is invalid"))
+                bunt::println!("{$red+bold}The JWT provided is invalid{/$}")
+            }
+            ErrorKind::InvalidSignature => {
+                bunt::eprintln!("{$red+bold}The JWT provided has an invalid signature{/$}")
+            }
+            ErrorKind::InvalidRsaKey => {
+                bunt::eprintln!("{$red+bold}The secret provided isn't a valid RSA key{/$}")
+            }
+            ErrorKind::InvalidEcdsaKey => {
+                bunt::eprintln!("{$red+bold}The secret provided isn't a valid ECDSA key{/$}")
             }
-            ErrorKind::InvalidSignature => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The JWT provided has an invalid signature",)
-            ),
-            ErrorKind::InvalidRsaKey => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The secret provided isn't a valid RSA key",)
-            ),
-            ErrorKind::InvalidEcdsaKey => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The secret provided isn't a valid ECDSA key",)
-            ),
             ErrorKind::ExpiredSignature => {
-                if should_validate_exp {
-                    println!("{}", Red.bold().paint("The token has expired"))
-                }
+                bunt::eprintln!("{$red+bold}The token has expired (or the `exp` claim is not set). This error can be ignored via the `--ignore-exp` parameter.{/$}")
             }
             ErrorKind::InvalidIssuer => {
-                println!("{}", Red.bold().paint("The token issuer is invalid"))
+                bunt::println!("{$red+bold}The token issuer is invalid{/$}")
             }
-            ErrorKind::InvalidAudience => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The token audience doesn't match the subject",)
+            ErrorKind::InvalidAudience => {
+                bunt::eprintln!("{$red+bold}The token audience doesn't match the subject{/$}")
+            }
+            ErrorKind::InvalidSubject => {
+                bunt::eprintln!("{$red+bold}The token subject doesn't match the audience{/$}")
+            }
+            ErrorKind::ImmatureSignature => bunt::eprintln!(
+                "{$red+bold}The `nbf` claim is in the future which isn't allowed{/$}"
             ),
-            ErrorKind::InvalidSubject => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The token subject doesn't match the audience",)
+            ErrorKind::InvalidAlgorithm => bunt::eprintln!(
+                "{$red+bold}The JWT provided has a different signing algorithm than the one you \
+                     provided{/$}",
             ),
-            ErrorKind::ImmatureSignature => eprintln!(
-                "{}",
-                Red.bold()
-                    .paint("The `nbf` claim is in the future which isn't allowed",)
-            ),
-            ErrorKind::InvalidAlgorithm => eprintln!(
-                "{}",
-                Red.bold().paint(
-                    "The JWT provided has a different signing algorithm than the one you \
-                     provided",
-                )
-            ),
-            _ => eprintln!(
-                "{} {:?}",
-                Red.bold().paint("The JWT provided is invalid because"),
+            _ => bunt::eprintln!(
+                "{$red+bold}The JWT provided is invalid because{/$} {:?}",
                 err
             ),
         };
     }
 
     match (format, token_data) {
-        (OutputFormat::JSON, Ok(token)) => {
+        (OutputFormat::Json, Ok(token)) => {
             println!("{}", to_string_pretty(&TokenOutput::new(token)).unwrap())
         }
         (_, Ok(token)) => {
-            println!("\n{}", Plain.bold().paint("Token header\n------------"));
+            bunt::println!("\n{$bold}Token header\n------------{/$}");
             println!("{}\n", to_string_pretty(&token.header).unwrap());
-            println!("{}", Plain.bold().paint("Token claims\n------------"));
+            bunt::println!("{$bold}Token claims\n------------{/$}");
             println!("{}", to_string_pretty(&token.claims).unwrap());
         }
         (_, Err(_)) => exit(1),
     }
 
     exit(match validated_token {
-        Err(err) => match (err.kind(), should_validate_exp) {
-            (ErrorKind::ExpiredSignature, false) => 0,
-            _ => 1,
-        },
+        Err(_) => 1,
         Ok(_) => 0,
     })
 }
@@ -633,14 +633,14 @@ fn main() {
 
     match matches.subcommand() {
         ("encode", Some(encode_matches)) => {
-            warn_unsupported(&encode_matches);
+            warn_unsupported(encode_matches);
 
-            let token = encode_token(&encode_matches);
+            let token = encode_token(encode_matches);
 
             print_encoded_token(token);
         }
         ("decode", Some(decode_matches)) => {
-            let (validated_token, token_data, format) = decode_token(&decode_matches);
+            let (validated_token, token_data, format) = decode_token(decode_matches);
 
             print_decoded_token(validated_token, token_data, format);
         }
