@@ -3,12 +3,13 @@ include!("../src/main.rs");
 #[cfg(test)]
 mod tests {
     use super::{
-        config_options, create_header, decode_token, encode_token, is_payload_item,
-        is_timestamp_or_duration, translate_algorithm, OutputFormat, Payload, PayloadItem,
-        SupportedAlgorithms,
+        config_options, create_header, decode_token, decoding_key_from_secret, encode_token,
+        encoding_key_from_secret, is_payload_item, is_timestamp_or_duration, translate_algorithm,
+        OutputFormat, Payload, PayloadItem, SupportedAlgorithms,
     };
+    use base64::decode as base64_decode;
     use chrono::{Duration, TimeZone, Utc};
-    use jsonwebtoken::{Algorithm, Header, TokenData};
+    use jsonwebtoken::{Algorithm, DecodingKey, EncodingKey, Header, TokenData};
     use serde_json::{from_value, json};
 
     fn empty_args() -> impl IntoIterator<Item = String> {
@@ -97,6 +98,18 @@ mod tests {
             SupportedAlgorithms::from_string("RS512"),
             SupportedAlgorithms::RS512
         );
+        assert_eq!(
+            SupportedAlgorithms::from_string("PS256"),
+            SupportedAlgorithms::PS256
+        );
+        assert_eq!(
+            SupportedAlgorithms::from_string("PS384"),
+            SupportedAlgorithms::PS384
+        );
+        assert_eq!(
+            SupportedAlgorithms::from_string("PS512"),
+            SupportedAlgorithms::PS512
+        );
         assert_eq!(
             SupportedAlgorithms::from_string("yolo"),
             SupportedAlgorithms::HS256
@@ -157,6 +170,18 @@ mod tests {
             translate_algorithm(SupportedAlgorithms::RS512),
             Algorithm::RS512
         );
+        assert_eq!(
+            translate_algorithm(SupportedAlgorithms::PS256),
+            Algorithm::PS256
+        );
+        assert_eq!(
+            translate_algorithm(SupportedAlgorithms::PS384),
+            Algorithm::PS384
+        );
+        assert_eq!(
+            translate_algorithm(SupportedAlgorithms::PS512),
+            Algorithm::PS512
+        );
     }
 
     #[test]
@@ -370,6 +395,45 @@ mod tests {
         assert_eq!(exp_claim.unwrap(), exp);
     }
 
+    #[test]
+    fn returns_error_when_exp_is_not_set() {
+        let encode_matcher = config_options()
+            .get_matches_from_safe(vec!["jwt", "encode", "-S", "1234567890"])
+            .unwrap();
+        let encode_matches = encode_matcher.subcommand_matches("encode").unwrap();
+        let encoded_token = encode_token(&encode_matches).unwrap();
+        let decode_matcher = config_options()
+            .get_matches_from_safe(vec!["jwt", "decode", "-S", "1234567890", &encoded_token])
+            .unwrap();
+        let decode_matches = decode_matcher.subcommand_matches("decode").unwrap();
+        let (decoded_token, _, _) = decode_token(&decode_matches);
+
+        assert!(decoded_token.is_err());
+    }
+
+    #[test]
+    fn returns_no_error_when_ignore_exp_parameter_is_set() {
+        let encode_matcher = config_options()
+            .get_matches_from_safe(vec!["jwt", "encode", "-S", "1234567890"])
+            .unwrap();
+        let encode_matches = encode_matcher.subcommand_matches("encode").unwrap();
+        let encoded_token = encode_token(&encode_matches).unwrap();
+        let decode_matcher = config_options()
+            .get_matches_from_safe(vec![
+                "jwt",
+                "decode",
+                "-S",
+                "1234567890",
+                "--ignore-exp",
+                &encoded_token,
+            ])
+            .unwrap();
+        let decode_matches = decode_matcher.subcommand_matches("decode").unwrap();
+        let (decoded_token, _, _) = decode_token(&decode_matches);
+
+        assert!(decoded_token.is_ok());
+    }
+
     #[test]
     fn allows_for_a_custom_exp_as_systemd_string() {
         let encode_matcher = config_options()
@@ -474,7 +538,7 @@ mod tests {
         let (result, _, format) = decode_token(&decode_matches);
 
         assert!(result.is_ok());
-        assert!(format == OutputFormat::JSON);
+        assert!(format == OutputFormat::Json);
     }
 
     #[test]
@@ -559,7 +623,7 @@ mod tests {
     }
 
     #[test]
-    fn encodes_and_decodes_an_rsa_token_using_key_from_file() {
+    fn encodes_and_decodes_an_rsa_ssa_pkcs1_v1_5_token_using_key_from_file() {
         let body: String = "{\"field\":\"value\"}".to_string();
         let encode_matcher = config_options()
             .get_matches_from_safe(vec![
@@ -592,6 +656,93 @@ mod tests {
         assert!(result.is_ok());
     }
 
+    #[test]
+    fn encoding_key_from_secret_handles_at() {
+        let expected = EncodingKey::from_secret(include_bytes!("hmac-key.bin"));
+        let key = encoding_key_from_secret(&Algorithm::HS256, "@./tests/hmac-key.bin").unwrap();
+        assert_eq!(expected, key);
+    }
+
+    #[test]
+    fn encoding_key_from_secret_handles_base64() {
+        let b64 = "+t0vs/PPB0dvyYKIk1DYvz5WyCUds5DLy07ycOK5oHA=";
+        let arg = format!("b64:{}", b64);
+        let expected = EncodingKey::from_secret(&base64_decode(b64).unwrap());
+        let key = encoding_key_from_secret(&Algorithm::HS256, &arg).unwrap();
+        assert_eq!(expected, key);
+    }
+
+    #[test]
+    fn decoding_key_from_secret_handles_at() {
+        let expected = DecodingKey::from_secret(include_bytes!("hmac-key.bin"));
+        let key = decoding_key_from_secret(&Algorithm::HS256, "@./tests/hmac-key.bin").unwrap();
+        assert_eq!(expected, key);
+    }
+
+    #[test]
+    fn decoding_key_from_secret_handles_base64() {
+        let b64 = "+t0vs/PPB0dvyYKIk1DYvz5WyCUds5DLy07ycOK5oHA=";
+        let arg = format!("b64:{}", b64);
+        let expected = DecodingKey::from_secret(&base64_decode(b64).unwrap()).into_static();
+        let key = decoding_key_from_secret(&Algorithm::HS256, &arg).unwrap();
+        assert_eq!(expected, key);
+    }
+
+    #[test]
+    fn encodes_and_decodes_an_rsa_ssa_pss_token_using_key_from_file() {
+        let body: String = "{\"field\":\"value\"}".to_string();
+        let encode_matcher = config_options()
+            .get_matches_from_safe(vec![
+                "jwt",
+                "encode",
+                "-A",
+                "PS256",
+                "--exp",
+                "-S",
+                "@./tests/private_rsa_key.der",
+                &body,
+            ])
+            .unwrap();
+        let encode_matches = encode_matcher.subcommand_matches("encode").unwrap();
+        let encoded_token = encode_token(&encode_matches).unwrap();
+        println!("enc {}", encoded_token);
+        let decode_matcher = config_options()
+            .get_matches_from_safe(vec![
+                "jwt",
+                "decode",
+                "-S",
+                "@./tests/public_rsa_key.der",
+                "-A",
+                "PS256",
+                &encoded_token,
+            ])
+            .unwrap();
+        let decode_matches = decode_matcher.subcommand_matches("decode").unwrap();
+        let (result, _, _) = decode_token(&decode_matches);
+
+        assert!(result.is_ok());
+    }
+
+    #[test]
+    fn decodes_an_rsa_ssa_pss_token_using_key_from_file() {
+        let token: String = "eyJ0eXAiOiJKV1QiLCJhbGciOiJQUzUxMiJ9.eyJmaWVsZCI6InZhbHVlIiwiaWF0IjoxNjI1OTMxNjAwLCJleHAiOjkwMDAwMDAwMDB9.Tt1siDczvVAi89dH8QqTZ_n5Ejz4gAIzVLqucWN5tEqdAVRdWgP8psuRFdC8RKIn1Lp4OsUkAA7NJ79cZt32Eewy84hTYrCgZZ9mcWg5IfXPHcZmTUm6qSyKqANdsnRWThbG3IJSX1D6obI5Y91NhVI5PTRg8sFlDAXaNN9ZVTmAtZXj0b5-MgsjiRqWMW3xi9xQqTxvb5VN37Oot-KDWZXjkO022ixshzFWu8Jt582uMD4qYRp1d0VldgyGO_viDqqk8qTqNA7soUKWyDds0emuecE_bDMeELMfxMR-A1pQeu3FgEhliazIAdXJMNlwRuJG8znLNqCK1nB2Nd8sUQ".to_string();
+        let decode_matcher = config_options()
+            .get_matches_from_safe(vec![
+                "jwt",
+                "decode",
+                "-S",
+                "@./tests/public_rsa_key.der",
+                "-A",
+                "PS512",
+                &token,
+            ])
+            .unwrap();
+        let decode_matches = decode_matcher.subcommand_matches("decode").unwrap();
+        let (result, _, _) = decode_token(&decode_matches);
+
+        assert!(result.is_ok());
+    }
+
     #[test]
     fn encodes_and_decodes_an_ecdsa_token_using_key_from_file() {
         let body: String = "{\"field\":\"value\"}".to_string();
